#########################################################################################
# Exonmap is a Package to provide fine grained annotation of Affymetrix exon microarrays.
# It makes use of a MySQL database containing the annotation
# 
# Please see the README.TXT file in this installation.
#
# Authors: M. Okoniewski, C.J. Miller
#########################################################################################

#### Basic function to load data, similar to simpleaffy

read.exon <- function(covdesc="covdesc",path=".",...) {
  samples <- read.AnnotatedDataFrame( paste(path,covdesc,sep="/"));
  files.to.read <- rownames(pData(samples));
  files.to.read <- paste(path,files.to.read,sep="/")
  eset <- ReadAffy(filenames=files.to.read,...);
  newPhenoData <- cbind(pData(eset),pData(samples)[rownames(pData(eset)),]);
  colnames(newPhenoData) <- c(colnames(pData(eset)),colnames(pData(samples)));
  tmp <- as.list(colnames(newPhenoData));
  names(tmp) <- colnames(newPhenoData);
  newPhenoData <- as(newPhenoData,"AnnotatedDataFrame")
  
  phenoData(eset) <- newPhenoData;	
  return(eset);
}


###
# The following functions take a character vector of names, connect to the database, request the appropriate translation,
# and return the mapped data.

#######################################################################################
probeset.to.gene <- function(X,list.out=FALSE, vector.out=TRUE,symbols.out=FALSE, unique=TRUE, drop.mt=FALSE, debug=0) 
{
out <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    if(drop.mt) probeset.list[[i]] <- exclude.probewise(probeset.list[[i]] , filter="multitarget") 
    out <- rbind(out, .probeset.to.gene(probeset.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"probeset"])
   else if (symbols.out) out <- unique(out[,"display_label"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}


#######################################################################################
.probeset.to.gene <- function(X, debug=0) 
{
pp <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_probesetToGene('",pp,"')",sep="")
out <-  .doStoredProcedureFetch( proc )
if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
probeset.to.exon <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,  drop.mt=FALSE, debug=0) 
{
out <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    if(drop.mt) probeset.list[[i]] <- exclude.probewise(probeset.list[[i]] , filter="multitarget") 
    out <- rbind(out, .probeset.to.exon(probeset.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"probeset"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}

#######################################################################################
.probeset.to.exon <- function(X, debug=0) 
{
pp <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_probesetToExon('",pp,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
probeset.to.transcript <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,  drop.mt=FALSE, debug=0) 
{
out <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    if(drop.mt) probeset.list[[i]] <- exclude.probewise(probeset.list[[i]] , filter="multitarget") 
    out <- rbind(out, .probeset.to.transcript(probeset.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"probeset"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}

#######################################################################################
.probeset.to.transcript <- function(X, debug=0) 
{
pp <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_probesetToTranscript('",pp,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
probeset.to.probe <- function(X, list.out=FALSE,  drop.mt=FALSE, debug=0) 
{
out <- NULL
selected <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n")
    if(drop.mt) probeset.list[[i]] <- exclude.probewise(probeset.list[[i]] , filter="multitarget") 
    if(debug>6) cat (probeset.list[[i]],"\n")
    out <- rbind(out, .probeset.to.probe(probeset.list[[i]], debug=debug ) )
  }
if (list.out) out <- split(out, out[,"probeset_name"])
out
}
#######################################################################################
.probeset.to.probe <- function(X, debug=0)
{
pp <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_probesetToHit('",pp,"')",sep="")
data <-  .doStoredProcedureFetch( proc )

out <- NULL
if(debug >6) cat (dim(data))
if(dim(data)[1]>0)  out <- data[,]
out
}
#######################################################################################
exon.to.transcript <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,  debug=0) 
{
out <- NULL
exon.list <- .split.vector(X, debug=debug)
for (i in 1:length(exon.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .exon.to.transcript(exon.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"exon"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}
#######################################################################################
.exon.to.transcript <- function(X, debug=0) 
{
ee <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_exonToTranscript('",ee,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
exon.to.probeset <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,  probes.min=4, debug=0) 
{
out <- NULL
exon.list <- .split.vector(X, debug=debug)
for (i in 1:length(exon.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .exon.to.probeset(exon.list[[i]], probes.min=probes.min, debug=debug))
  }
if (list.out) out <- split(out[,"probeset_name"], out[,"exon"])
       else if (vector.out) out <- unique(out[,"probeset_name"])
out
}
#######################################################################################
.exon.to.probeset <- function(X, probes.min=4,  debug=0) 
{
ee <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_exonToProbeset('",ee,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
else if (dim(out)[1]>0) out <- out[out[,"num_probes"]>=probes.min,]
out
}
#######################################################################################
exon.to.gene <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,  debug=0) 
{
out <- NULL
exon.list <- .split.vector(X, debug=debug)
for (i in 1:length(exon.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .exon.to.gene(exon.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"exon"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}
#######################################################################################
.exon.to.gene <- function(X, debug=0) 
{
ee <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_exonToGene('",ee,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
transcript.to.gene <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,   debug=0) 
{
out <- NULL
transcript.list <- .split.vector(X, debug=debug)
for (i in 1:length(transcript.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .transcript.to.gene(transcript.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"transcript"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}
#######################################################################################
.transcript.to.gene <- function(X, debug=0) 
{
tt <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_transcriptToGene('",tt,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
transcript.to.exon <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,   debug=0) 
{
out <- NULL
transcript.list <- .split.vector(X, debug=debug)
for (i in 1:length(transcript.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .transcript.to.exon(transcript.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"transcript"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}
#######################################################################################
.transcript.to.exon <- function(X, debug=0) 
{
tt <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_transcriptToExon('",tt,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
transcript.to.probeset <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,  probes.min=4,  debug=0) 
{
out <- NULL
transcript.list <- .split.vector(X, debug=debug)
for (i in 1:length(transcript.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .transcript.to.probeset(transcript.list[[i]], probes.min=probes.min, debug=debug))
  }
if (list.out) out <- split(out[,"probeset_name"], out[,"transcript"])
       else if (vector.out) out <- unique(out[,"probeset_name"])
out
}
#######################################################################################
.transcript.to.probeset <- function(X, probes.min=4, debug=0) 
{
tt <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_transcriptToProbeset('",tt,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
else if(dim(out)[1]>0) out <- out[out[,"probe_count"]>=probes.min,]
  
out
}
#######################################################################################
gene.to.transcript <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,   debug=0) 
{
out <- NULL
gene.list <- .split.vector(X, debug=debug)
for (i in 1:length(gene.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .gene.to.transcript(gene.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"gene"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}
#######################################################################################
.gene.to.transcript <- function(X, debug=0) 
{
gg <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_geneToTranscript('",gg,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
gene.to.exon <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE,   debug=0) 
{
out <- NULL
gene.list <- .split.vector(X, debug=debug)
for (i in 1:length(gene.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .gene.to.exon(gene.list[[i]], debug=debug))
  }
if (list.out) out <- split(out[,"stable_id"], out[,"gene"])
       else if (vector.out) out <- unique(out[,"stable_id"])
out
}
#######################################################################################
.gene.to.exon <- function(X, debug=0) 
{
gg <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_geneToExon('",gg,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
gene.to.probeset <- function(X,list.out=FALSE, vector.out=TRUE, unique=TRUE, probes.min=4,  debug=0) 
{
out <- NULL
gene.list <- .split.vector(X, debug=debug)
for (i in 1:length(gene.list))
  {
    if(debug>5) cat ("Processing ",i,"th piece\n");
    out <- rbind(out, .gene.to.probeset(gene.list[[i]], probes.min=probes.min, debug=debug))
  }
if (list.out) out <- split(out[,"probeset_name"], out[,1])
       else if (vector.out) out <- unique(out[,"probeset_name"])
out
}
#######################################################################################
.gene.to.probeset <- function(X, probes.min=4, debug=0) 
{
gg <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
proc <- paste ("CALL xmap_geneToProbeset('",gg,"')",sep="")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
else if(dim(out)[1]>0) out <- out[out[,"probe_count"]>=probes.min,]
out
}
#######################################################################################
symbol.to.gene <- function(X, debug=0) 
{
ss <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}

if (length(grep("homo",.ExonmapDBName))>0) num.spe=1100
if (length(grep("mus",.ExonmapDBName))>0) num.spe=1400
if (length(grep("rattus",.ExonmapDBName))>0) num.spe=2800
proc <- paste ("CALL xmap_externalDbidToGene(",num.spe,",'",ss,"')",sep="")
if (debug>8) cat(proc,"\n")
out <-  .doStoredProcedureFetch( proc )

if(debug >6) cat (dim(out))
if(dim(out)[1]<=0) out <- NULL
out
}
#######################################################################################
symbol.to.probeset <- function(X, vector.out=TRUE, db=c("hugo","symbol","UniProt","RefSeqPeptide","RFAM","UniProtSpliceVariant","miRNA"),probes.min=4, debug=0)
{
gg <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}

db <- match.arg(db)
proc <- switch(db,
     "hugo"                 = paste("CALL xmap_externalDbidToProbeset(1100,'",gg,"')",sep=""), 
     "symbol"               = paste("CALL xmap_externalDbidToProbeset(1100,'",gg,"')",sep=""),
     "UniProt"              = paste("CALL xmap_externalDbidToProbeset(2000,'",gg,"')",sep=""),
     "RefSeqPeptide"        = paste("CALL xmap_externalDbidToProbeset(1815,'",gg,"')",sep=""),
     "RFAM"                 = paste("CALL xmap_externalDbidToProbeset(4200,'",gg,"')",sep=""),
     "UniProtSpliceVariant" = paste("CALL xmap_externalDbidToProbeset(2202,'",gg,"')",sep=""),
     "miRNA"                = paste("CALL xmap_externalDbidToProbeset(3300,'",gg,"')",sep=""))
data <-  .doStoredProcedureFetch( proc )

out <- NULL
if(dim(data)[1]>0)
  {
    idx <-  which (data[,4]>=probes.min)
    if(vector.out) out <- data[idx,3]
    else out <- data[idx,]
  }
out
}      

#######################################################################################
.exclude.probesets <- function(X,filter=c("nonexonic","intronic","intergenic","multitarget"), debug=0) 
{
out <- NULL
selected <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>4) cat ("Processing ",i,"th piece\n")
    selected <- c(selected, .select.X(probeset.list[[i]],filter=filter))
  }
out <- setdiff(X , selected)
out
}
#######################################################################################
# old .select.probesets
.select.X <- function(X,filter=c("nonexonic","intronic","intergenic","multitarget")) 
{
out <- NULL
   list.in <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
   proc <- paste ("CALL xmap_probesetStats('",list.in,"')",sep="")
   data <-  .doStoredProcedureFetch( proc )

   if(dim(data)[1]>0)
    {
      if(filter=="intronic") idx <- which(data[,"total_distinct_genes"]>0 & data[,"total_distinct_exons"]==0)
      if(filter=="exonic") idx <- which(data[,"total_distinct_genes"]>0 & data[,"total_distinct_exons"]>1)
      if(filter=="plainexonic") idx <- which(data[,"total_distinct_genes"]==1 & data[,"total_distinct_exons"]==1)
      if(filter=="plainintronic") idx <- which(data[,"total_distinct_genes"]==1 & data[,"total_distinct_exons"]==0)
      if(filter=="plain") idx <- which(data[,"total_distinct_genes"]==1)
      if(filter=="nonexonic") idx <- which(data[,"total_distinct_exons"]==0)
      if(filter=="intergenic") idx <- which(data[,"total_distinct_genes"]==0)
      if(filter=="multitarget") idx <- which( data[,"total_distinct_genes"]>1 | data[,"total_distinct_exons"]>1 )
    }
out <- data[idx,"probeset"]
out
}
#######################################################################################
.select.probesets <- function(X,filter=c("nonexonic","intronic","intergenic","multitarget"), debug=0) 
{
out <- NULL
selected <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>4) cat ("Processing ",i,"th piece\n")
    out <- c(out, .select.X(probeset.list[[i]],filter=filter))
  }
out
}
#######################################################################################
exclude.probewise <- function(X,filter=c("exonic","intronic","intergenic","multitarget"), mt.level=1, debug=0) 
{
out <- NULL
selected <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>4) cat ("Processing ",i,"th piece\n")
    selected <- c(selected, .select.probewise(probeset.list[[i]],filter=filter, mt.level=mt.level))
  }
out <- setdiff(X , selected)
out
}
#######################################################################################
.select.probewise <- function(X,filter=c("exonic","intronic","intergenic","multitarget","nulltarget"), mt.level=1, debug=0) 
{
out <- NULL
   list.in <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
   proc <- paste ("CALL xmap_probesetSummary('",list.in,"')",sep="")
   data <-  .doStoredProcedureFetch( proc )

   if(debug >1) cat ("Size of outdata:",dim(data),"\n")
   if(dim(data)[1]>0)
    {   
      if(filter=="intronic") out <- data[(data[,"hitScore"]==1 & data[,"exonScore"]==0  & data[,"geneScore"]>0),"probeset"]
      if(filter=="exonic") out <- data[(data[,"hitScore"]==1 & data[,"exonScore"]>0  & data[,"geneScore"]>0),"probeset"]
      if(filter=="intergenic") out <- data[(data[,"hitScore"]==1 & data[,"exonScore"]==0  & data[,"geneScore"]==0),"probeset"]
      if(filter=="multitarget") out <- data[which( data[,"hitScore"]>mt.level),"probeset"]
      if(filter=="nulltarget") out <- data[( data[,"hitScore"]==0),"probeset"]
    }
out
}
#######################################################################################
select.probewise <- function(X,filter=c("exonic","intronic","intergenic","multitarget"), mt.level=1, debug=0) 
{
out <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>4) cat ("Processing ",i,"th piece\n")
    out <- c(out, .select.probewise(probeset.list[[i]],filter=filter, mt.level=mt.level))
  }
out
}
#######################################################################################
.probeset.stats <- function(X)
{
   pp <- .vector.to.commalist(X)
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
   proc <- paste ("CALL xmap_probesetSummary('",pp,"')",sep="")
   data <-  .doStoredProcedureFetch( proc )

   data
}
#######################################################################################
probeset.stats <- function(X, debug=0)
{
out <- NULL
probeset.list <- .split.vector(X, debug=debug)
for (i in 1:length(probeset.list))
  {
    if(debug>4) cat ("Processing ",i,"th piece\n")
    out <- rbind(out, .probeset.stats(probeset.list[[i]]))
  }
out
}
#######################################################################################
probesets.in.range <- function(start, stop, strand, chr, debug=0)
{
out <- NULL
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}

proc <- paste ("CALL xmap_probesetsInRange(",start,",",stop,",",strand,",'", chr,"')",sep="")
data <-  .doStoredProcedureFetch( proc )

if(dim(data)[1]>0) out <- data[,"name"]
out
}

#######################################################################################
genes.in.range <- function(start, stop, strand, chr, debug=0)
{
out <- NULL
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}

proc <- paste ("CALL xmap_genesInRange(",start,",",stop,",",strand,",'", chr,"')",sep="")
data <-  .doStoredProcedureFetch( proc )

if(dim(data)[1]>0) out <- data
out
}
#######################################################################################
probes.only.in.range <- function(start, stop, strand, chr, debug=0)
{
out <- NULL
if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}

proc <- paste ("CALL xmap_probesOnlyInRange(",start,",",stop,",",strand,",'", chr,"')",sep="")
data <-  .doStoredProcedureFetch( proc )

if(dim(data)[1]>0) out <- data
out
}

#######################################################################################
gene.details <- function(X) {
  out <- NULL
  if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
  data <-  sapply(X,function(a) {
        proc <- paste("CALL xmap_geneDetails('",a,"')",sep="")

	r    <- .doStoredProcedureFetch( proc )

        if(dim(r)[1] == 0) { stop(paste("Could not find gene details for '",a,"'.",sep=""))}
        r
  })
  data <- t(data)
  rownames(data) <- X
  if(dim(data)[1]>0) out <- as.data.frame(data)
  out
}

######################################################################################
transcript.details <- function(X) {
  out <- NULL
  if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
  data <-  sapply(X,function(a) {
        proc <- paste("CALL xmap_transcriptDetails('",a,"')",sep="")

	r    <- .doStoredProcedureFetch( proc )

        if(dim(r)[1] == 0) { stop(paste("Could not find transcript details for '",a,"'.",sep=""))}
        r
  })
  data <- t(data)
  rownames(data) <- X
  if(dim(data)[1]>0) out <-  as.data.frame(data)
  out
}

#######################################################################################
exon.details <- function(X) {
  out <- NULL
  if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
  data <- c()
  for(i in X) {
    proc <- paste("CALL xmap_exonDetails('",i,"')",sep="")

    r    <- .doStoredProcedureFetch( proc )

    if(dim(r)[1] == 0) { stop(paste("Could not find exon details for '",a,"'.",sep=""))}
    data <- rbind(data,r)
  }
  if(dim(data)[1]>0) out <-  as.data.frame(data)
  out
}

#######################################################################################
probesets.in.eset <- function(X,data) {
  if(class(data) != "ExpressionSet") { stop("data should be an ExpressionSet") }
  intersect(X,rownames(exprs(data)))
}

#######################################################################################
# utility functions - hidden
.vector.to.commalist <- function(v)
{
out <- v[1]
if(length(v)>1)
{
  for (i in 2:length(v))
  {
    out <- paste(out,v[i], sep=",")
  }
}
out
}


.doStoredProcedureFetch <- function( sql ) {
  .xmapConnect()
  if(!exists(".ExonmapConnection")) {
	stop("Not connected to database!")
  }
  else {
    connection <- .ExonmapConnection
  }
  res <- dbSendQuery( connection, sql )
  data <- fetch( res, n=-1 )
  repeat {
    fetch( res, n=-1 )
    if( dbHasCompleted( res ) ) break ;
  }
  dbClearResult(res)
  .xmapDisconnect()
  data
}

.split.vector <- function(v, debug=0)
{
out <- NULL
if(length(v)>0)
{
extrabit <- 0
elt.size <- max(nchar(v))
if(debug>5) cat("elt.size", elt.size,"\n")
list.size <- trunc(65000/(elt.size+1))
if(debug>5) cat("list.size", list.size,"\n")
num.lists <- trunc(length(v)/list.size)
if(debug>5) cat("num.lists", num.lists,"\n")
if((length(v)%%list.size)>0) {extrabit <- 1}
 if(num.lists>0)
 {
   for (i in 0:(num.lists-1))
   {
     if(debug>8) cat(i) 
     out[[i+1]]<- v[((i*list.size)+1):((i+1)*(list.size))]
   }
 }
if(debug>5) cat("extrabit ", extrabit,"\n")
if(extrabit==1) out[[num.lists+1]] <- v[(num.lists*list.size+1):(length(v))]
}
else out[[1]] <- v
out
}

######################################################################################
# free up any results set if a function terminates unexpectedy (e.g. by ctrl-c)

.cleanup.rs <- function() {
  if(exists(".ExonmapConnection")) {
    connection <- .ExonmapConnection
    res <- dbSendQuery( connection, sql )
    data <- fetch( res, n=-1 )
    repeat {
      fetch( res, n=-1 )
      if( dbHasCompleted( res ) ) break ;
    }
    dbClearResult(res)
  }
}

#######################################################################################
# Database switching
xmapDatabase <- function(db=c("Human", "Mouse"))
{
  db <- match.arg(db)
  switch(db,
    "Human" = .dbHuman(),
    "Mouse" = .dbMouse())
  
  #.xmapConnect()
  cat("done.\n");
}

.xmapConnect <- function()
{
  con <- dbConnect("MySQL",user=.ExonmapUser, password=.ExonmapPassword, dbname=.ExonmapDBName, host=.ExonmapHost, client.flag="196608" )
  assign(".ExonmapConnection",con,envir=.GlobalEnv);
}


.xmapDisconnect <- function() {
  if(exists(".ExonmapConnection")) {
   dbDisconnect(.ExonmapConnection)
   #remove(.ExonmapConnection)
  }
}

.dbMouse <- function()
{
if(file.exists("~/.exonmap/mouse.mycnf"))
  {
   conf <- read.table("~/.exonmap/mouse.mycnf", row.names=1)
   assign(".ExonmapUser",as.character(conf["user",]),envir=.GlobalEnv);
   assign(".ExonmapPassword", as.character(conf["password",]),envir=.GlobalEnv);
   assign(".ExonmapDBName",as.character(conf["database",]),envir=.GlobalEnv);
   assign(".ExonmapHost",as.character(conf["host",]),envir=.GlobalEnv);
   cat("Switching to mouse database...\n")
  }
else stop("No  ~/.exonmap/mouse.mycnf file found!\n")
}

.dbHuman <- function()
{
if(file.exists("~/.exonmap/human.mycnf"))
  {
    conf <- read.table("~/.exonmap/human.mycnf", row.names=1)
    assign(".ExonmapUser",as.character(conf["user",]),envir=.GlobalEnv);
    assign(".ExonmapPassword", as.character(conf["password",]),envir=.GlobalEnv);
    assign(".ExonmapDBName",as.character(conf["database",]),envir=.GlobalEnv);
    assign(".ExonmapHost",as.character(conf["host",]),envir=.GlobalEnv);
    cat("Switching to human database...\n")
  }
else stop("No  ~/.exonmap/human.mycnf file found!\n")
}

showXmapDatabase <- function()
{
cat("User",.ExonmapUser,"\n" )
cat("Password",.ExonmapPassword,"\n" )
cat("Database",.ExonmapDBName,"\n")
cat("Host",.ExonmapHost,"\n")
}



