\name{probeset.to.probe}
\alias{probeset.to.probe}
\alias{probeset.to.exon}
\alias{probeset.to.transcript}
\alias{probeset.to.gene}
\alias{exon.to.probeset}
\alias{exon.to.transcript}
\alias{exon.to.gene}
\alias{transcript.to.gene}
\alias{transcript.to.exon}
\alias{transcript.to.probeset}
\alias{gene.to.transcript}
\alias{gene.to.exon}
\alias{gene.to.probeset}
\alias{symbol.to.probeset}
\alias{symbol.to.gene}

\title{ Use the X:MAP database to translate between genes, transcripts, exons, probesets and probes }
\description{
Each of the functions is in the form X.to.Y, and takes a list of identifiers for the things of type X and returns the corresponding things of type Y.

}
\usage{
  probeset.to.probe(X,list.out=FALSE, drop.mt=FALSE,  debug=0)
  probeset.to.exon(X,list.out=FALSE, vector.out=TRUE, unique=TRUE, drop.mt=FALSE, debug=0) 
  probeset.to.transcript(X, list.out=FALSE, vector.out=TRUE, unique=TRUE, drop.mt=FALSE, debug=0) 
  probeset.to.gene(X,list.out=FALSE, vector.out=TRUE,symbols.out=FALSE, unique=TRUE, drop.mt=FALSE, debug=0) 
  exon.to.probeset(X, list.out=FALSE, vector.out=TRUE, unique=TRUE, probes.min=4, debug=0) 
  exon.to.transcript(X,list.out=FALSE, vector.out=TRUE, unique=TRUE, debug=0) 
  exon.to.gene(X,list.out=FALSE, vector.out=TRUE, unique=TRUE, debug=0) 
  transcript.to.exon(X, list.out=FALSE,vector.out=TRUE, unique=TRUE, debug=0) 
  transcript.to.probeset(X, list.out=FALSE,vector.out=TRUE, unique=TRUE, probes.min=4,  debug=0)
  transcript.to.gene (X, list.out=FALSE, vector.out=TRUE, unique=TRUE, debug=0) 
  gene.to.exon(X, list.out=FALSE, vector.out=TRUE, unique=TRUE, debug=0)   
  gene.to.probeset(X, list.out=FALSE, vector.out=TRUE, unique=TRUE, probes.min=4,  debug=0) 
  symbol.to.probeset(X, vector.out=TRUE,db=c("hugo","symbol","UniProt","RefSeqPeptide","RFAM","UniProtSpliceVariant","miRNA"),probes.min=4, debug=0)
  symbol.to.gene(X, debug=0) 
}

\arguments{
  \item{X}{ Vector of identifiers to map (e.g. an ensembl gene id) }
  \item{list.out}{ If TRUE, results are presented as a list. Overrides effects of vector.out and symbols.out }
  \item{vector.out}{ By default, return only the names of the mapped identifiers, if FALSE, return a summary table }
  \item{symbols.out}{ If TRUE, returns just gene symbols }
  \item{unique}{ If TRUE, pre-filters the data to remove dublicates }
  \item{drop.mt}{ Filter out multi-targeting probesets prior to database search }
  \item{probes.min}{ Only return probesets that hit the genome with at least this number of probes }
  \item{db}{ The name of the symbol database to search when looking for the Ensembl gene identifier }
  \item{debug}{ nything other than 0 returns debugging information. Verbosity increases with value }
  
}
\details{ }
\value{
  Vector of probeset names, or a data frame with probeset names and number of probe hits.
}
\references{\url{ http://bioinformatics.picr.man.ac.uk/}}
\author{ Michal Okoniewski }

\keyword{ misc }
\examples{ 
  \dontrun{
    genes <- probeset.to.gene("2326601"); 
    genes <- probeset.to.gene(c("2326780","2326822" )); 
    genes <- probeset.to.gene(c("2326780","2326822"), vector.out=FALSE, probes.min=1); # produces a table of results, with genes matched by the probesets. 
  }
}
