\name{gene.strip}
\alias{gene.strip}

\title{ Use the X:MAP database to find annotated gene structure and generate a plot for multiple genes}
\description{
Takes a list of genes and an \code{ExpressionSet} object and generates
a plot. Each row corresponds to a gene, X axis corresponds to
position. By default each exon is ploted as a rectangle, coloured by
expression (see below), and introns are ignored. Overlapping exons are
plotted next to each other, if multiple probesets hit an exon they are
stacked. Data are filtered to remove multiply targeted probesets. If
there are no 'well-behaved' probesets hitting an exon it is drawn as a
white rectangle.

If plot.introns is TRUE then introns are included in the plot, and
position on the x-axis is relative to the start of the gene. Exons are
drawn as rectangles in the border colour - default is black.  Note
that they will show up as verticle lines, if the gene is long and the
data are compressed to fit into the space available for the plot. Each
probe is represented by a circle (in the border colour; default is
black) and a line which is coloured by expression.

}
\usage{
gene.strip(X,data,gp1,gp2,plot.introns=FALSE,labels,type=c("mean-fc","min-fc","max-fc","min-tt","mean-int","median-int","min-int","max-int"),
           scale.to.gene=FALSE,palette=col.rd.bl,col.range=c(-5,5),use.only.wbex=TRUE,exclude.mt=TRUE,
           cex=0.5,xlab,ylab,border="black",plot=TRUE)
}

\arguments{
\item{X}{the Ensembl gene id of the gene to plot}
\item{data}{Expression data (should be an \code{ExpressionSet}). Used to colour the plot}
\item{gp1}{indices into the expression data; group 1 for pairwise fold-change and t-test calculations}
\item{gp2}{indices into the expression data; group 2 for pairwise fold-change and t-test calculations}
\item{plot.introns}{Changes style of the graph to show both exons and introns}
\item{labels}{Labels for the gene defaults to the Ensembl gene ID, the gene name and a '>' or '<' indicating whether the gene is on the forward or reverse strand}
\item{type}{The calculation used to generate the colouring}
\item{scale.to.gene}{If true, colouring is relative to the average expression for the gene}
\item{palette}{Colours used to represent expression}
\item{col.range}{Numerical extent of the colouring}
\item{use.only.wbex}{Ignore probesets that don't hit the genome in one place, within an exon}
\item{exclude.mt}{If use.only.wbex is FALSE then itron probesets are used if plot.introns=TRUE. exclude.mt will filter this list to remove probesets that hit the genome at multiple sites}
\item{cex}{character size}
\item{xlab}{x axis label}
\item{ylab}{y axis label}
\item{border}{colour to paint edges of exons and, if plot.introns is TRUE,  probe locations}
\item{plot}{Just set up axes, don't plot whole graph. Useful because large graphs can take a long time to plot; this parameter makes it possible to check labels etc. are as expected.}
}
\details{

 At its simplest, takes an ensembl gene id and plots the location and
 structure of the gene.  If data, gp1, and gp2 are specified, then
 colours the gene according to the expression data.  By default, this
 is done by calculating the mean fold change for all the well behaved
 exon probes (i.e. those that only hit the genome, once, in an exon in
 the gene of interest), mapping this value into the chosen palette and
 using the resulting colour to paint the gene. The same is done for
 transcripts and exons.  Other methods of colouring are specified by
 \code{type}, and should be self-explanatory. See the vignette for
 more details.  If scale.to.gene is TRUE, then fold-changes (or
 intensities, depending on the value of \code{type}) are calculated
 relative to the fold change for the gene.  

 Note that for fold change calculations the number returned is (gp1 -
 gp2) i.e. if gp1 is more highly expressed than group 2, the result is
 positive. With default colouring, positive values are blue, negative,
 red.
  
}
\value{
  none
}
\seealso{\code{\link{gene.legend}}}

\references{\url{ http://bioinformatics.picr.man.ac.uk/}}
\author{ Crispin Miller }

\keyword{ misc }
\examples{ 
  \dontrun{
     plot.gene("ENSG00000141510")
  }
}
