\name{plot.gene}
\alias{plot.gene}
\alias{col.rd.bl}

\title{ Use the X:MAP database to find annotated gene structure and generate a plot }
\description{
  Draws a plot of a gene's structure, possibly coloured by expression data, similar to
  that found on the X:MAP website

}
\usage{
plot.gene(x,data,gp1,gp2,xlim,ylim,tick.labels=T,
          border="black",
          show.probes=T,probe.colour="black",show.mt=T,mt.colour="grey",
          type=c("mean-fc","min-fc","max-fc","min-tt","mean-int","median-int","min-int","max-int"),
          use.only.wbex=T,palette=col.rd.bl,col.range,scale.to.gene=F,draw.exon.border=T,colour.transcript=F,
          draw.legend=T,cex=0.5,...)
col.rd.bl
}

\arguments{
\item{x}{the Ensembl gene id of the gene to plot}
\item{data}{Expression data (should be an \code{ExpressionSet}). If present, used to colour the plot}
\item{gp1}{indices into the expression data; group 1 for pairwise fold-change and t-test calculations}
\item{gp2}{indices into the expression data; group 2 for pairwise fold-change and t-test calculations}
\item{xlim}{chromosomal coordinates for the plot. If missing defaults to the position of the gene}
\item{ylim}{height of the plot}
\item{tick.labels}{label ticks?}
\item{border}{colour to draw gene/transcript/exon edges and text}
\item{show.probes}{draw individual probes?}
\item{probe.colour}{colour for well-behaving probes}
\item{show.mt}{draw multiply targeted probes}
\item{mt.colour}{colour to draw multiply targeted probes}
\item{type}{how to colour by expression data}
\item{use.only.wbex}{only use well behaving exon probes for expression colouring}
\item{palette}{colour scheme for expression colouring}
\item{col.range}{min aand maximum values (e.g. fold-change) to map to the colour palette }
\item{scale.to.gene}{pre-scale exon colours to the value for the gene before plotting }
\item{draw.exon.border}{draw an edge round the exons?}
\item{colour.transcript}{colour individual transcripts by expression?}
\item{draw.legend}{add a colour bar to the plot?}
\item{cex}{character expansion}
\item{...}{additional parameters to pass through to \code{plot}}
}
\details{

 At its simplest, takes an ensembl gene id and plots the location and
 structure of the gene.  If data, gp1, and gp2 are specified, then
 colours the gene according to the expression data.  By default, this
 is done by calculating the mean fold change for all the well behaved
 exon probes (i.e. those that only hit the genome, once, in an exon in
 the gene of interest), mapping this value into the chosen palette and
 using the resulting colour to paint the gene. The same is done for
 transcripts and exons.  Other methods of colouring are specified by
 \code{type}, and should be self-explanatory. See the vignette for
 more details.  If scale.to.gene is TRUE, then fold-changes (or
 intensities, depending on the value of \code{type}) are calculated
 relative to the fold change for the gene.  

 Note that for fold change calculations the number returned is (gp1 -
 gp2) i.e. if gp1 is more highly expressed than group 2, the result is
 positive. With default colouring, positive values are blue, negative,
 red.
  
}
\value{
  none
}
\seealso{\code{\link{gene.legend}}}

\references{\url{ http://bioinformatics.picr.man.ac.uk/}}
\author{ Crispin Miller }

\keyword{ misc }
\examples{ 
  \dontrun{
     plot.gene("ENSG00000141510")
  }
}
