\name{plot.gene.graph}
\alias{plot.gene.graph}

\title{ Use the X:MAP database to find annotated gene structure and generate a plot }
\description{
  Draws a plot of a gene's structure, with a line graph showing expression over the gene's length for two sets of samples }
}
\usage{

plot.gene.graph(x,data,gp1,gp2,xlim,ylim,
                border="black",
                show.mt=T,mt.colour="grey",
                type=c("mean-fc","min-fc","max-fc","min-tt","mean-int","median-int","min-int","max-int"),
                palette=col.rd.bl,col.range=c(-5,5),use.only.wb=T,scale.to.gene=F,
                gp1.col="orange",
                gp2.col="red",
                draw.exon.border=T,exon.height=0.05,draw.legend=T,cex=0.5,xlab,ylab="",...)
}

\arguments{
\item{x}{The name of the gene to plot}
\item{data}{ExpressionSet containing expression data to map to gene}
\item{gp1}{indices into the expression set for the first replicate group}
\item{gp2}{indices into the expression set for the second replicate group}
\item{xlim}{range specifying position on the gene's chromosome to plot. Defaults to the location of the gene}
\item{ylim}{range specifying y axis extent}
\item{border}{colour for probesets exon borders etc.}
\item{show.mt}{plot probesets that hit the genome more than once}
\item{mt.colour}{colour to draw mt probesets}
\item{type}{how to summarize data for thhe replicate groups}
\item{palette}{vector of colours to use to colour the exons}
\item{col.range}{range specifying the numeric extent of the colouring}
\item{use.only.wb}{only use well behaving probesets to calculate the exon summaries}
\item{scale.to.gene}{ calculate the summary value for the entire gene (e.g. mean fold change, mean intensity, ...) and scale all values relative to that before plotting}
\item{gp1.col}{group 1 colour}
\item{gp2.col}{group 2 colour}
\item{draw.exon.border}{draw a box round the exons?}
\item{exon.height}{how high to draw the exons}
\item{draw.legend}{add a legend to the plot?}
\item{cex}{label text expansion}
\item{xlab}{x axis label}
\item{ylab}{y axis label}
\item{...}{extra parameters to pass through to plot}
}
\details{
 At its simplest, takes an ensemble gene id and plots the location of
 the gene and its exons.  Data, gp1, and gp2 are used to colour the
 exons according to the expression data, and to draw line plots for
 each of the two groups.  By default, this is done by calculating the
 mean fold change for all the well behaved exon probes (i.e. those
 that only hit the genome, once, in an exon in the gene of interest),
 mapping this value into the chosen palette and using the resulting
 colour to paint the exon. At the same time a line graph is produced
 for the individual probesets.  Other methods of colouring are
 specified by \code{type}, and should be self-explanatory. See the
 vignette for more details.  If scale.to.gene is TRUE, then
 fold-changes (or intensities, depending on the value of \code{type})
 are calculated relative to that for the gene.  

 Note that for fold change calculations the number returned is (gp1 -
 gp2) i.e. if gp1 is more highly expressed than group 2, the result is
 positive. With default colouring, positive values are blue, negative,
 red.
 
}
\value{
  none
}
\seealso{\code{\link{plot.gene}}}

\references{\url{ http://bioinformatics.picr.man.ac.uk/}}
\author{ Crispin Miller }

\keyword{ misc }
\examples{ 
  \dontrun{
     plot.gene.graph("ENSG00000141510",x.rma,1:3,4:6)
  }
}
