\name{splicing.index}
\alias{splicing.index}

\title{ Find probesets involved in differential splicing for two treatments}
\description{
 Reads expression data and using its \code{\link[Biobase]{pData}}, 
 calculates splicing index for each probeset in exons of genes from the list. 
 The splicing index formula come from the Affy white
 paper "Alternative Transcript Analysis Methods for Exon Arrays".
}
\usage{
splicing.index(x, genes, group, members, vector.out=T, debug=0)
}

\arguments{
  \item{x}{eSet for which splicing is to be calculated }
  \item{genes}{ Vector of Ensembl gene names to check }
  \item{group}{ Group name from covdesc }
  \item{members}{Two treatments to be chosen from the group }
  \item{vector.out}{ Vector or tabular output }
  \item{debug}{ Debug level - 0 stands for no debug messages, verbosity increases with the level}
}
\value{
  A vector of values of slicing index for each probeset - the further from 0, the more significant splicing. 
  Or a table with names of genes, exon, probesets, expression and splicing index.
}
\details{Splicing index operates on expression values of probesets normalized 
with gene-level expression. The value of the index may have values from -1 to 1.}
\references{\url{ http://bioinformatics.picr.man.ac.uk/}}
\author{ Michal J Okoniewski }

\seealso{ \code{\link[exonmap]{splanova}}, \code{\link[Biobase]{pData}  }}
\keyword{ misc }
\examples{ 
  \dontrun{
    gg <- probeset.to.gene(c("2326780","2326822" ))
    spl.idx <-  splicing.index(x.rma, gg, "group", c("a","b"), debug=4)
  }
}
